@@ -1,12 +1,11 @@
 #![allow(non_snake_case)]
 
 use bitcoin::consensus::{deserialize, encode, Decodable};
-use bitcoin::SigHashType;
+use bitcoin::EcdsaSighashType;
 use blocks_iterator::{Config, PeriodCounter};
 use env_logger::Env;
 use log::info;
 use std::error::Error;
-use std::sync::mpsc::sync_channel;
 use std::time::Duration;
 use structopt::StructOpt;
 
@@ -15,14 +14,13 @@ fn main() -> Result<(), Box<dyn Error>> {
     info!("start");
 
     let config = Config::from_args();
-    let (send, recv) = sync_channel(config.channels_size.into());
-    let handle = blocks_iterator::iterate(config, send);
+    let iter = blocks_iterator::iter(config);
     let mut signatures_in_witness = 0;
     let mut block_with_witness;
     let mut blocks_with_witness = 0;
     let mut period = PeriodCounter::new(Duration::from_secs(10));
 
-    while let Some(block_extra) = recv.recv()? {
+    for block_extra in iter {
         block_with_witness = false;
         if period.period_elapsed().is_some() {
             info!(
@@ -37,7 +35,7 @@ fn main() -> Result<(), Box<dyn Error>> {
 
         for tx in block_extra.block.txdata {
             for input in tx.input {
-                for witness in input.witness {
+                for witness in input.witness.iter() {
                     if let Ok(_sig) = deserialize::<ParsedSignature>(&witness) {
                         signatures_in_witness += 1;
                         block_with_witness = true;
@@ -49,7 +47,6 @@ fn main() -> Result<(), Box<dyn Error>> {
             blocks_with_witness += 1;
         }
     }
-    handle.join().expect("couldn't join");
     info!(
         "signatures_in_witness: {} blocks_with_witness: {}",
         signatures_in_witness, blocks_with_witness
@@ -58,13 +55,14 @@ fn main() -> Result<(), Box<dyn Error>> {
 }
 
 struct ParsedSignature {
-    pub _sighash: SigHashType,
+    pub _sighash: EcdsaSighashType,
     pub _R: Vec<u8>,
     pub _s: Vec<u8>,
 }
 
 impl Decodable for ParsedSignature {
     fn consensus_decode<D: std::io::Read>(mut d: D) -> Result<Self, encode::Error> {
+        //TODO fix for schnorr signatures!
         let first = u8::consensus_decode(&mut d)?;
         if first != 0x30 {
             return Err(encode::Error::ParseFailed("Signature must start with 0x30"));
@@ -82,7 +80,7 @@ impl Decodable for ParsedSignature {
         }
         let s = <Vec<u8>>::consensus_decode(&mut d)?;
         let sighash_u8 = u8::consensus_decode(&mut d)?;
-        let sighash = SigHashType::from_u32_consensus(sighash_u8 as u32);
+        let sighash = EcdsaSighashType::from_consensus(sighash_u8 as u32);
 
         Ok(ParsedSignature {
             _sighash: sighash,
