@@ -3,6 +3,11 @@
 //! Read bitcoin blocks directory containing `blocks*.dat` files, and produce a ordered stream
 //! of [BlockExtra]
 //!
+//! Blocks could be consumed in 2 different ways:
+//! * Through the [`iter`] or [`par_iter`] method. The latter is to be preferred when performing
+//!   costly task.
+//! * Via unix pipeline using the [`PipeIterator`]
+//!
 
 // Coding conventions
 #![forbid(unsafe_code)]
@@ -23,6 +28,7 @@ use log::{error, info, Level};
 use std::fs::File;
 
 use std::path::PathBuf;
+use std::sync::atomic::AtomicBool;
 use std::sync::mpsc::{sync_channel, Receiver, SyncSender};
 use std::sync::{Arc, Mutex};
 use std::thread;
@@ -82,6 +88,10 @@ pub struct Config {
     /// Reduce the memory requirements but it's slower and use disk space
     #[structopt(short, long)]
     pub utxo_db: Option<PathBuf>,
+
+    /// Stop the blocks iteration at the specified height
+    #[structopt(long)]
+    pub stop_at_height: Option<u32>,
 }
 
 impl Config {
@@ -100,8 +110,9 @@ impl Config {
 
 /// Before reorder we keep only the position of the block in the file system and data relative
 /// to the block hash, the previous hash and the following hash (populated during reorder phase)
-/// We will need to read the block from disk again, but by doing so we will avoid using too much
-/// memory in the [`OutOfOrderBlocks`] map.
+/// We will need
+///  to read the block from disk again, but by doing so we will avoid using too much
+/// memory in the `OutOfOrderBlocks` map.
 #[derive(Debug)]
 pub struct FsBlock {
     /// the file the block identified by `hash` is stored in. Multiple blocks are stored in the
@@ -128,42 +139,6 @@ pub struct FsBlock {
     pub next: Vec<BlockHash>,
 }
 
-/// Read `blocks*.dat` contained in the `config.blocks_dir` directory and returns [BlockExtra]
-/// through a channel supplied from the caller. Blocks returned are ordered from the genesis to the
-/// highest block in the directory (minus `config.max_reorg`).
-/// In this call threads are spawned, caller must call [std::thread::JoinHandle::join] on the returning handle.
-pub fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandle<()> {
-    thread::spawn(move || {
-        let now = Instant::now();
-
-        // FsBlock is a small struct (~120b), so 10_000 is not a problem but allows the read_detect to read ahead the next block file
-        let (send_block_fs, receive_block_fs) = sync_channel(0);
-        let _read =
-            read_detect::ReadDetect::new(config.blocks_dir.clone(), config.network, send_block_fs);
-
-        let (send_ordered_blocks, receive_ordered_blocks) =
-            sync_channel(config.channels_size.into());
-        let send_ordered_blocks = if config.skip_prevout {
-            // if skip_prevout is true, we send directly to end step
-            channel.clone()
-        } else {
-            send_ordered_blocks
-        };
-        let _reorder = reorder::Reorder::new(
-            config.network,
-            config.max_reorg,
-            receive_block_fs,
-            send_ordered_blocks,
-        );
-
-        if !config.skip_prevout {
-            let _fee = fee::Fee::new(receive_ordered_blocks, channel, config.utxo_manager());
-        }
-
-        info!("Total time elapsed: {}s", now.elapsed().as_secs());
-    })
-}
-
 struct BlockExtraIterator {
     handle: Option<JoinHandle<()>>,
     recv: Receiver<Option<BlockExtra>>,
@@ -202,8 +177,106 @@ pub fn iter(config: Config) -> impl Iterator<Item = BlockExtra> {
     BlockExtraIterator { handle, recv }
 }
 
+#[cfg(feature = "parallel")]
+/// `par_iter` is used when the task to be performed on the blockchain is more costly
+/// than iterating the blocks. For example verifying the spending conditions in the blockchain.
+/// like [`crate::iter`] accepts configuration parameters via the [`Config`] struct.
+/// A `PREPROC` closure has to be provided, this process a single block and produce a Vec of
+/// user defined struct `DATA`.
+/// A `TASK` closure accepts the `DATA` struct and a shared `STATE` and it is executed in a
+/// concurrent way. The `TASK` closure returns a bool which indicates if the execution should be
+/// terminated.
+/// Note that access to `STATE` in `TASK` should be done carefully otherwise the contention would
+/// reduce the speed of execution.
+///  
+pub fn par_iter<STATE, PREPROC, TASK, DATA>(
+    config: Config,
+    state: STATE,
+    pre_processing: PREPROC,
+    task: TASK,
+) where
+    PREPROC: Fn(BlockExtra) -> Vec<DATA> + 'static + Send,
+    TASK: Fn(DATA, Arc<STATE>) -> bool + 'static + Send + Sync,
+    DATA: 'static + Send,
+    STATE: 'static + Send + Sync,
+{
+    use log::debug;
+    use rayon::prelude::*;
+    use std::sync::atomic::Ordering::SeqCst;
+
+    let (send_task, recv_task) = sync_channel(config.channels_size.into());
+    let stop = Arc::new(AtomicBool::new(false));
+    let state = Arc::new(state);
+
+    let stop_clone = stop.clone();
+    let iter = iter(config);
+
+    let handle = thread::spawn(move || {
+        debug!("start pre-processing thread");
+        for block_extra in iter {
+            if stop_clone.load(SeqCst) {
+                break;
+            }
+            let data_vec = pre_processing(block_extra);
+            for data in data_vec {
+                send_task.send(data).unwrap();
+            }
+        }
+        debug!("ending pre-processing thread");
+    });
+
+    recv_task.into_iter().par_bridge().for_each(|data: DATA| {
+        debug!("iter");
+        let result = task(data, state.clone());
+
+        if result {
+            stop.store(true, SeqCst)
+        }
+    });
+    handle.join().unwrap();
+}
+
+fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandle<()> {
+    thread::spawn(move || {
+        let now = Instant::now();
+        let early_stop = Arc::new(AtomicBool::new(false));
+
+        // FsBlock is a small struct (~120b), so 10_000 is not a problem but allows the read_detect to read ahead the next block file
+        let (send_block_fs, receive_block_fs) = sync_channel(0);
+        let _read = read_detect::ReadDetect::new(
+            config.blocks_dir.clone(),
+            config.network,
+            early_stop.clone(),
+            send_block_fs,
+        );
+
+        let (send_ordered_blocks, receive_ordered_blocks) =
+            sync_channel(config.channels_size.into());
+        let send_ordered_blocks = if config.skip_prevout {
+            // if skip_prevout is true, we send directly to end step
+            channel.clone()
+        } else {
+            send_ordered_blocks
+        };
+        let _reorder = reorder::Reorder::new(
+            config.network,
+            config.max_reorg,
+            config.stop_at_height,
+            early_stop,
+            receive_block_fs,
+            send_ordered_blocks,
+        );
+
+        if !config.skip_prevout {
+            let _fee = fee::Fee::new(receive_ordered_blocks, channel, config.utxo_manager());
+        }
+
+        info!("Total time elapsed: {}s", now.elapsed().as_secs());
+    })
+}
+
 /// Utility method usually returning [log::Level::Debug] but when `i` is divisible by `every` returns [log::Level::Info]
-#[deprecated]
+#[deprecated(note = "use `period::Periodic` or `period::PeriodCounter`")]
 pub fn periodic_log_level(i: u32, every: u32) -> Level {
     if i % every == 0 {
         Level::Info
@@ -228,6 +301,7 @@ mod inner_test {
             channels_size: 0,
             #[cfg(feature = "db")]
             utxo_db: None,
+            stop_at_height: None,
         };
         let (send, recv) = sync_channel(0);
 
@@ -250,6 +324,7 @@ mod inner_test {
             skip_prevout: false,
             max_reorg: 10,
             channels_size: 0,
+            stop_at_height: None,
             utxo_db: Some(tempdir.path().to_path_buf()),
         };
         let (send, recv) = sync_channel(0);
